# Generated by Django 5.0.14 on 2025-06-26 15:06
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('wbcompliance', '0020_rename_trigger_content_type_riskcheck_checked_object_content_type_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='riskcheck',
            name='activator_content_type',
            field=models.ForeignKey(blank=True, max_length=256, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='riskcheck',
            name='activator_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
