# Generated by Django 5.0.9 on 2024-10-11 11:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcompliance", "0016_checkedobjectincidentrelationship_report_details_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="rulebackend",
            name="incident_report_template",
            field=models.TextField(
                default='<ul style="line-height: 13px; text-align: left; margin: 0; padding: 10px;">\n    {% for key, value in report_details.items %}\n        <li><strong>{{ key }}: </strong> {{ value | safe }}</li>\n    {% endfor %}\n</ul>\n',
                verbose_name="Incident Template",
            ),
        ),
    ]
